# Generated by roxygen2: do not edit by hand

S3method(as.dendrogram,aibclust)
S3method(as.hclust,aibclust)
S3method(coef,aibclust)
S3method(coef,gibclust)
S3method(coef,sibclust)
S3method(fitted,aibclust)
S3method(fitted,gibclust)
S3method(fitted,sibclust)
S3method(info_metrics,aibclust)
S3method(info_metrics,gibclust)
S3method(info_metrics,sibclust)
S3method(plot,aibclust)
S3method(plot,gibclust)
S3method(plot,sibclust)
S3method(predict,gibclust)
S3method(predict,sibclust)
S3method(print,aibclust)
S3method(print,gibclust)
S3method(print,sibclust)
S3method(print,summary.aibclust)
S3method(print,summary.gibclust)
S3method(print,summary.sibclust)
S3method(summary,aibclust)
S3method(summary,gibclust)
S3method(summary,sibclust)
export(AIBmix)
export(DIBmix)
export(GIBmix)
export(IBmix)
export(find_elbow)
export(info_metrics)
export(sIBmix)
import(Rdpack)
import(np)
import(stats)
importFrom(Rcpp,evalCpp)
importFrom(grDevices,adjustcolor)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,hcl.colors)
importFrom(graphics,axis)
importFrom(graphics,barplot)
importFrom(graphics,box)
importFrom(graphics,contour)
importFrom(graphics,image)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,rect)
importFrom(graphics,segments)
importFrom(graphics,text)
importFrom(rje,powerSet)
importFrom(utils,flush.console)
importFrom(utils,head)
importFrom(utils,setTxtProgressBar)
importFrom(utils,tail)
importFrom(utils,txtProgressBar)
useDynLib(IBclust, .registration = TRUE)
