#' IBclust: Information-Based Clustering
#'
#' @useDynLib IBclust, .registration = TRUE
#' @import stats
#' @import np
#' @import Rdpack
#' @importFrom utils head flush.console txtProgressBar setTxtProgressBar
#' @importFrom Rcpp evalCpp
#' @importFrom rje powerSet
#' @importFrom graphics barplot points
"_PACKAGE"